/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ExtendedJettyWebService.java 3141 2006-08-31 03:11:20Z jmettraux $
 */

//
// JettyWebService.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.webserver;

import org.mortbay.http.SocketListener;
import org.mortbay.util.InetAddrPort;
import org.mortbay.jetty.Server;

import openwfe.org.AbstractService;
import openwfe.org.ServiceException;
import openwfe.org.ApplicationContext;
import openwfe.org.xml.XmlUtils;


/**
 * A Service that creates and binds a WebServerImpl.
 * This service has no useful methods : all its work is done in the 'init'
 * method.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-08-31 05:11:20 +0200 (Thu, 31 Aug 2006) $
 * <br>$Id: ExtendedJettyWebService.java 3141 2006-08-31 03:11:20Z jmettraux $ </font>
 *
 * @author jmettraux@openwfe.org
 */
public class ExtendedJettyWebService

    extends AbstractService

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(JettyWebService.class.getName());

    //
    // CONSTANTS (definitions)

    public final static String PORT
        = "port";
    public final static String ADDRESS_AND_PORT
        = "addressAndPort";

    /**
     * use this key to point to a &lt;contextPath&gt;:&lt;directory&gt; pair
     */
    public final static String WEB
        = "web";

    public final static String LOG_FILE
        = "logFile";

    //
    // FIELDS

    private Server server = null;

    //
    // CONSTRUCTORS

    //
    // METHODS from Service

    public void init 
        (final String serviceName, 
         final ApplicationContext context, 
         final java.util.Map serviceParams)
    throws 
        ServiceException
    {
        super.init(serviceName, context, serviceParams);

        //
        // configure listener
        
        SocketListener listener = null;

        //
        // port

        String sPort = (String)serviceParams.get(PORT);
        if (sPort != null)
        {
            try
            {
                int port = Integer.parseInt(sPort);

                listener = new SocketListener();
                listener.setPort(port);

                log.info("Listening on port "+port);
            }
            catch (NumberFormatException nfe)
            {
                // ignore
            }
        }

        // 
        // address and port (takes precedence over 'port')
        
        String addressAndPort = (String)serviceParams.get(ADDRESS_AND_PORT);
        if (addressAndPort != null)
        {
            try
            {
                listener = new SocketListener(new InetAddrPort(addressAndPort));

                log.info("Listening on \""+addressAndPort+"\"");
            }
            catch (java.net.UnknownHostException uhe)
            {
                log.warn
                    ("Failed to listen on \""+addressAndPort+"\"", uhe);
                throw new ServiceException
                    ("Failed to listen on \""+addressAndPort+"\"", uhe);
            }
        }

        //
        // setup the servlet container

        this.server = new Server();
        this.server.addListener(listener);

        //
        // prepare webs
        
        Object oWebs = serviceParams.get(WEB);
        if (oWebs == null)
        {
            throw new ServiceException
                ("No webs specified in the parameters. Cannot start service.");
        }

        java.util.List lWebs = null;
        if (oWebs instanceof java.util.List)
        {
            lWebs = (java.util.List)oWebs;
        }
        else
        {
            lWebs = new java.util.ArrayList(1);
            lWebs.add(oWebs);
        }

        java.util.Iterator it = lWebs.iterator();
        while (it.hasNext()) addWeb((String)it.next());

        //
        // log file
        
        String logFileName = (String)serviceParams.get(LOG_FILE);

        if (logFileName == null) logFileName = "openwfe-jetty.log";

        System.setProperty("LOG_FILE", logFileName);

        //
        // start the server in a separate thread
        
        (new Thread ()
         {
             public void run ()
             {
                 try
                 {
                     ExtendedJettyWebService.this.server.start();
                 }
                 catch (Exception e)
                 {
                     JettyWebService.log.warn
                         ("Failed to start webserver", e);
                 }
             }
         }).start();
    }

    protected void addWeb (final String web)
    {
        log.debug("pwd : "+(new java.io.File(".")).getAbsolutePath());

        final String[] ss = web.split(":");

        final String webContext = ss[0];

        final String filePath = getContext().getApplicationDirectory() + ss[1];

        try
        {
            this.server.addWebApplication(webContext, filePath);

            log.info
                ("Added web \""+webContext+"\" -> \""+filePath+"\"");
        }
        catch (java.io.IOException ie)
        {
            log.warn
                ("Failed to add web \""+webContext+"\" -> \""+filePath+"\"", 
                 ie);
        }
    }

    public void stop ()
        throws ServiceException
    {
        super.stop();

        try
        {
            this.server.stop();
        }
        catch (InterruptedException ie)
        {
            throw new ServiceException
                ("Failed to stop servlet container.", ie);
        }
    }

    public org.jdom.Element getStatus ()
    {
        final org.jdom.Element result = new org.jdom.Element(getName());

        result.addContent(XmlUtils.getClassElt(this));
        result.addContent(XmlUtils.getRevisionElt("$Id: ExtendedJettyWebService.java 3141 2006-08-31 03:11:20Z jmettraux $"));

        return result;
    }

    //
    // METHODS

}
