/*
 * Copyright (c) 2005, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: JettyWebService.java 3141 2006-08-31 03:11:20Z jmettraux $
 */

//
// JettyWebService.java
//
// jmettraux@openwfe.org
//
// generated with 
// jtmpl 1.0.04 31.10.2002 John Mettraux (jmettraux@openwfe.org)
//

package openwfe.org.webserver;

import org.mortbay.http.SocketListener;
import org.mortbay.http.HttpServer;
import org.mortbay.http.HttpContext;
import org.mortbay.http.handler.ResourceHandler;
import org.mortbay.jetty.Server;

import openwfe.org.MapUtils;
import openwfe.org.AbstractService;
import openwfe.org.ServiceException;
import openwfe.org.ApplicationContext;
import openwfe.org.xml.XmlUtils;


/**
 * A Service that creates and binds a WebServerImpl.
 * This service has no useful methods : all its work is done in the 'init'
 * method.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author: jmettraux $
 * <br>$Date: 2006-08-31 05:11:20 +0200 (Thu, 31 Aug 2006) $
 * <br>$Id: JettyWebService.java 3141 2006-08-31 03:11:20Z jmettraux $ </font>
 *
 * @author jmettraux@openwfe.org
 */
public class JettyWebService

    extends AbstractService

{

    protected final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(JettyWebService.class.getName());

    //
    // CONSTANTS (definitions)

    /**
     * 'port' : on which port should this Jetty instance listen and serve.
     */
    public final static String P_PORT
        = "port";

    /**
     * 'baseDirectory' : the point in the filesystem where web content is 
     * stored.
     */
    public final static String P_BASE_DIRECTORY
        = "baseDirectory";

    /**
     * 'contextPath' : when this param is not set, the contextPath will be
     * "/";
     */
    public final static String P_CONTEXT_PATH
        = "contextPath";

    /**
     * 'staticContentOnly' : if this parameter is set to 'true', the jetty
     * service will only serve static pages, no servlets or JSP.
     */
    public final static String P_STATIC_CONTENT_ONLY
        = "staticContentOnly";

    //public final static String LOG_FILE
    //    = "logFile";

    //
    // FIELDS

    private HttpServer server = null;

    //
    // CONSTRUCTORS

    //
    // METHODS from Service

    public void init 
        (final String serviceName, 
         final ApplicationContext context, 
         final java.util.Map serviceParams)
    throws 
        ServiceException
    {
        super.init(serviceName, context, serviceParams);

        //
        // static content only
        //
        // (if set to true, only serves static content)
        
        final boolean staticContentOnly = MapUtils
            .getAsBoolean(getParams(), P_STATIC_CONTENT_ONLY, false);

        //
        // port

        int port = 8080;
        String sPort = (String)serviceParams.get(P_PORT);
        if (sPort != null)
        {
            try
            {
                port = Integer.parseInt(sPort);
            }
            catch (NumberFormatException nfe)
            {
                log.info
                    ("Cannot parse port '"+sPort+
                     "'. Falling back to "+port);
            }
        }

        log.info("Listening on port "+port);

        //
        // webClientPath
        
        String baseDirectory = (String)serviceParams.get(P_BASE_DIRECTORY);

        if (baseDirectory == null)
        {
            throw new ServiceException
                ("The parameter '"+P_BASE_DIRECTORY+
                 "' is required. Cancelling service.");
        }

        //
        // context path
        
        String contextPath = (String)serviceParams.get(P_CONTEXT_PATH);

        if (contextPath == null)
        {
            contextPath = "/";
        }

        log.info("ContextPath is set to '"+contextPath+"'");

        //
        // log file
        
        /*
        String logFileName = (String)serviceParams.get(LOG_FILE);

        if (logFileName == null) logFileName = "openwfe-jetty.log";

        System.setProperty("LOG_FILE", logFileName);

        //System.setProperty("LOG_LEVEL", "INFO");
            // no effect
        */

        //
        // setup the server

        if (staticContentOnly)
            //
            // setup the http server
        {
            this.server = new HttpServer();
            SocketListener listener = new SocketListener();
            listener.setPort(port);
            this.server.addListener(listener);
            HttpContext httpContext = new HttpContext();
            httpContext.setContextPath(contextPath);
            httpContext.setResourceBase(baseDirectory);
            httpContext.addHandler(new ResourceHandler());
            this.server.addContext(httpContext);
        }
        else
            //
            // setup the servlet container
        {
            this.server = new Server();
            SocketListener listener = new SocketListener();
            listener.setPort(port);
            this.server.addListener(listener);

            try
            {
                ((Server)this.server).addWebApplication
                    (contextPath, baseDirectory);
            }
            catch (Exception e)
            {
                throw new ServiceException
                    ("Failed to setup servlet container.", e);
            }
        }

        //
        // start the server in a separate thread
        
        (new Thread ()
         {
             public void run ()
             {
                 try
                 {
                     JettyWebService.this.server.start();
                 }
                 catch (Exception e)
                 {
                     JettyWebService.log.warn
                         ("Failed to start webserver", e);
                 }
             }
         }).start();
    }

    public void stop ()
        throws ServiceException
    {
        super.stop();

        try
        {
            this.server.stop();
        }
        catch (InterruptedException ie)
        {
            throw new ServiceException
                ("Failed to stop servlet container.", ie);
        }
    }

    public org.jdom.Element getStatus ()
    {
        final org.jdom.Element result = new org.jdom.Element(getName());

        result.addContent(XmlUtils.getClassElt(this));
        result.addContent(XmlUtils.getRevisionElt("$Id: JettyWebService.java 3141 2006-08-31 03:11:20Z jmettraux $"));

        return result;
    }

    //
    // METHODS

}
